// A state in the property automaton -*- c++ -*-

#ifndef PROPERTYSTATE_H_
# define PROPERTYSTATE_H_
# pragma interface

# include <cassert>

/** @file PropertyState.h
 * A state in the property automaton
 */

/** A state in the property automaton */
class PropertyState
{
public:
  /** Default constructor */
  PropertyState () :
    myNumSuccessors (0), mySuccessors (0), myGates (0) {}
private:
  /** Copy constructor */
  explicit PropertyState (const class PropertyState& old);
  /** Assignment operator */
  class PropertyState& operator= (const class PropertyState& old);
public:
  /** Destructor */
  ~PropertyState ();

  /** Determine the number of successor states */
  unsigned getNumSuccessors () const { return myNumSuccessors; }

  /** Access a successor
   * @param index	zero-based index number to the successor
   * @return		the successor
   */
  unsigned operator[] (unsigned index) const {
    assert (index < myNumSuccessors);
    return mySuccessors[index];
  }

  /** Add a successor state
   * @param num		the successor state number
   * @param gate	enabling condition
   */
  void addSuccessor (unsigned num,
		     class Expression& gate);

  /** Evaluate the gates
   * @param valuation	Valuation for evaluating the gates
   * @param result	(out) amount and numbers of enabled successors
   * @return		whether the evaluation was successful
   */
  bool eval (const class Valuation& valuation,
	     unsigned*& result) const;

# ifdef EXPR_COMPILE
  /** Compile the gate evaluator
   * @param cexpr	the compiled expression
   * @param indent	indentation level
   * @param result	name of the result vector
   */
  void compileGates (class CExpression& cexpr,
		     unsigned indent,
		     const char* result) const;
# endif // EXPR_COMPILE

private:
  /** Number of successor states */
  unsigned myNumSuccessors;
  /** The successor states */
  unsigned* mySuccessors;
  /** The successor gates */
  class Expression** myGates;
};

#endif // PROPERTYSTATE_H_
