// List of sets -*- c++ -*-

#pragma implementation
#include "SetList.h"
#include "BitVector.h"
#include <cstring>

/** @file SetList.C
 * A collection of sets of numbers
 */

SetList::SetList (unsigned numSets) :
  myNumSets (numSets), myAllocatedSets (0), mySets (0)
{
  for (myAllocatedSets = 1; myAllocatedSets < numSets; myAllocatedSets <<= 1);
  mySets = new unsigned*[myAllocatedSets];
  memset (mySets, 0, myAllocatedSets * sizeof *mySets);
}

SetList::~SetList ()
{
  for (unsigned i = myNumSets; i--; )
    delete[] mySets[i];
  delete[] mySets;
}

void
SetList::assign (unsigned index, unsigned* array)
{
  assert (index < myNumSets && array && *array);
  if (!mySets[index])
    mySets[index] = array;
  else {
    unsigned* sets = new unsigned[*mySets[index] + *array + 1];
    memcpy (sets, mySets[index], (*mySets[index] + 1) * sizeof *sets);
    memcpy (sets + *sets + 1, array + 1, *array * sizeof *sets);
    *sets += *array;
    delete[] mySets[index];
    delete[] array;
    mySets[index] = sets;
  }
}

void
SetList::assign(unsigned index,
		const class BitVector &bv,
		unsigned offset,
		bool complement)
{
  assert(index < myNumSets);

  if (!bv.getSize())
    return;

  unsigned numTrue = 0, i;
  for (i = bv.getSize(); i--; )
    if(bv[i] != complement) numTrue++;
  if (!numTrue)
    return;
  unsigned* s = new unsigned[numTrue + 1];
  *s = numTrue;

  for (i = bv.getSize();; )
    if(bv[--i] != complement)
      if ((s[numTrue] = i + offset), !--numTrue)
	break;
  assign (index, s);
}

void
SetList::copy (unsigned index, const unsigned* array)
{
  assert (index < myNumSets && array && *array);
  if (!mySets[index]) {
    mySets[index] = new unsigned[*array + 1];
    memcpy (mySets[index], array, (*array + 1) * sizeof *array);
  }
  else {
    unsigned* sets = new unsigned[*mySets[index] + *array + 1];
    memcpy (sets, mySets[index], (*mySets[index] + 1) * sizeof *sets);
    memcpy (sets + *sets + 1, array + 1, *array * sizeof *sets);
    *sets += *array;
    delete[] mySets[index];
    mySets[index] = sets;
  }
}

void
SetList::grow (unsigned numSets)
{
  assert (numSets > myNumSets);
  if (numSets <= myAllocatedSets) {
    memset (mySets + myNumSets, 0, (numSets - myNumSets) * sizeof *mySets);
    myNumSets = numSets;
    return;
  }
  for (; myAllocatedSets < numSets; myAllocatedSets <<= 1);
  unsigned** sets = new unsigned*[myAllocatedSets];
  memcpy (sets, mySets, myNumSets * sizeof *sets);
  memset (sets + myNumSets, 0, (numSets - myNumSets) * sizeof *sets);
  delete[] mySets;
  mySets = sets;
  myNumSets = numSets;
}

void
SetList::shrink (unsigned numSets)
{
  assert (numSets <= myNumSets);
  while (myNumSets > numSets)
    delete[] mySets[--myNumSets];
}
