/* coding/decoding base definitions */

#include<stdlib.h>
#include<string.h>

/* determine the endianness */
#if defined __alpha
# include <machine/endian.h>
#elif defined __sun
# include <sys/byteorder.h>
# if !defined BYTE_ORDER
#  define LITTLE_ENDIAN 1234
#  define BIG_ENDIAN 4321
#  if defined _LITTLE_ENDIAN
#   define BYTE_ORDER LITTLE_ENDIAN
#  elif defined _BIG_ENDIAN
#   define BYTE_ORDER BIG_ENDIAN
#  endif /* little/big endian */
# endif /* !defined BYTE_ORDER */
#else
# include <endian.h>
#endif

/** @file runtime/codec.h
 * Definitions for encoding and decoding values
 */

/* Machine word */
#if (BYTE_ORDER == BIG_ENDIAN || BYTE_ORDER == LITTLE_ENDIAN)
/** Machine word */
typedef unsigned long word_t;
#else /* mixed-endian */
/** Machine word */
typedef unsigned char word_t;
#endif /* BYTE_ORDER */
#define WORD_T_BIT (CHAR_BIT * sizeof (word_t))

/** number of words a specified amount of bits takes */
#define NUM_WORDS(bits) (((bits) + (WORD_T_BIT - 1)) / WORD_T_BIT)
/** number of bytes a specified amount of bits takes */
#define NUM_BYTES(bits) (((bits) + (CHAR_BIT - 1)) / CHAR_BIT)
/** number of words a specified amount of bytes takes */
#define NUM_WORD_B(bytes) (((bytes) + (sizeof (word_t) - 1)) / sizeof (word_t))

/* determine whether CHAR_BIT is a power of 2 */
#if (CHAR_BIT - 1) & CHAR_BIT
/** Round up a bit amount to the nearest multiple of the machine word length */
# define ROUNDUP_WORDS(bits) (NUM_WORDS(bits) * WORD_T_BIT)
/** Round up a bit amount to the nearest multiple of byte length */
# define ROUNDUP_BYTES(bits) (NUM_BYTES(bits) * CHAR_BIT)
#else
/** Round up a bit amount to the nearest multiple of the machine word length */
# define ROUNDUP_WORDS(bits) (((bits) + (WORD_T_BIT - 1)) & ~(WORD_T_BIT - 1))
/** Round up a bit amount to the nearest multiple of byte length */
# define ROUNDUP_BYTES(bits) (((bits) + (CHAR_BIT - 1)) & ~(CHAR_BIT - 1))
#endif

/** encoding variable */
#define ENCVAR(x) FCN(enc##_##x)
/** decoding variable */
#define DECVAR(x) FCN(dec##_##x)
