// set of variables -*- c++ -*-

#ifndef VARIABLESET_H_
# define VARIABLESET_H_
# pragma interface

# include <set>

/** @file VariableSet.h
 * Set of variables, a transient class used in preprocessing
 */

/** Set of variables */
class VariableSet
{
public:
  /** Variable set */
  typedef std::set<const class VariableDefinition*> Set;
  /** Constant iterator to the variable set */
  typedef Set::const_iterator const_iterator;
  /** Constructor */
  VariableSet () : mySet () {}
  /** Copy constructor */
  explicit VariableSet (const class VariableSet& old) : mySet (old.mySet) {}
private:
  /** Assignment operator */
  class VariableSet& operator= (const class VariableSet& old);
public:
  /** Destructor */
  ~VariableSet () {}

  /** Determine whether the set contains a variable
   * @param var		variable to search
   * @return		whether the set contains the variable
   */
  bool contains (const class VariableDefinition& var) const;
  /** Insert a variable to the set
   * @param var		variable to insert
   */
  void insert (const class VariableDefinition& var);

  /** @name Accessors to the variable set */
  /*@{*/
  void clear () { mySet.clear (); }
  bool empty () const { return mySet.empty (); }
  size_t size () const { return mySet.size (); }
  const_iterator begin () const { return mySet.begin (); }
  const_iterator end () const { return mySet.end (); }
  /*@}*/

private:
  /** The set of variables */
  Set mySet;
};

#endif // VARIABLESET_H_
