// Global marking (PlaceMarking for each place) -*- c++ -*-

#ifndef GLOBALMARKING_H_
# define GLOBALMARKING_H_
# pragma interface

# include "PlaceMarking.h"

/** @file GlobalMarking.h
 * Assignment of markings for each place
 */

/** Global marking (PlaceMarking for each place) */
class GlobalMarking
{
public:
  /** Constructor */
  explicit GlobalMarking (const class Net& net);
  /** Copy constructor */
  explicit GlobalMarking (const class GlobalMarking& old);
private:
  /** Assignment operator */
  class GlobalMarking& operator= (const class GlobalMarking& old);
public:
  /** Destructor */
  ~GlobalMarking ();

# ifndef NDEBUG
  /** Get the net associated with the marking */
  const class Net& getNet () const { return myNet; }
# endif // NDEBUG
  /** Get the number of place markings */
  unsigned getSize () const { return mySize; }

  /** Get a PlaceMarking by place number
   * @param index	the place number
   */
  class PlaceMarking& operator[] (unsigned index) {
    assert (index < getSize ());
    return myPlaceMarkings[index];
  }
  /** Get a PlaceMarking by place number
   * @param index	the place number
   */
  const class PlaceMarking& operator[] (unsigned index) const {
    return (*const_cast<class GlobalMarking*>(this))[index];
  }

  /** Get a PlaceMarking by place
   * @param place       the place
   */
  class PlaceMarking& operator[] (const class Place& place);
  /** Get a PlaceMarking by place
   * @param place       the place
   */
  const class PlaceMarking& operator[] (const class Place& place) const {
    return (*const_cast<class GlobalMarking*>(this))[place];
  }

  /** Flag the state erroneous */
  void flagErroneous () { myErroneous = true; }
  /** Determine whether the state is erroneous */
  bool isErroneous () const { return myErroneous; }

# ifndef NDEBUG
  /** Equality comparison
   * @param other	object to be compared with
   * @return		true if the objects are equal
   */
  bool operator== (const class GlobalMarking& other) const {
    assert (&myNet == &other.myNet && mySize == other.mySize);
    for (unsigned i = 0; i < mySize; i++)
      if (!(myPlaceMarkings[i] == other.myPlaceMarkings[i]))
	return false;
    return true;
  }
# endif // NDEBUG

  /** Display this object
   * @param printer	The printer object
   */
  void display (const class Printer& printer) const;

private:
# ifndef NDEBUG
  /** The net this marking belongs to */
  const class Net& myNet;
# endif // NDEBUG
  /** Flag: is the marking erroneous (is an erroneous transition enabled?) */
  bool myErroneous;
  /** Number of place markings (or places in the net) */
  unsigned mySize;
  /** The place markings */
  class PlaceMarking* myPlaceMarkings;
};

#endif // GLOBALMARKING_H_
