// Place marking class -*- c++ -*-

#pragma implementation
#include "PlaceMarking.h"
#include "Place.h"
#ifndef NDEBUG
# include "Type.h"
#endif
#include <cassert>
#include <cstring>

/** @file PlaceMarking.C
 * Assignment of a place to a multi-set
 */

#ifndef NDEBUG
void
PlaceMarking::setPlace (const class Place* place)
{
  assert (!place || !myPlace);
  if (place)
    setType (&place->getType ());
  myPlace = place;
}

void
PlaceMarking::setType (const class Type* type)
{
  assert (!myPlace && !myType);
  myType = type;
}
#endif // NDEBUG

bool
PlaceMarking::add (class Value& value, card_t amount)
{
  assert (myType->isConstrained (value));
  assert (amount > 0);
  std::pair<iterator, bool> result =
    myTokens.insert (TokenMap::value_type (&value, amount));
  if (!result.second) {
    delete &value;
    if (amount >= CARD_T_MAX - result.first->second)
      return false;
    result.first->second += amount;
  }
  return true;
}

void
PlaceMarking::remove (const class Value& value, card_t amount)
{
  assert (myType->isConstrained (value));
  assert (amount > 0);
  iterator i = find (&value);
  assert (i != end ());
  assert (i->second >= amount);
  i->second -= amount;
}

#include "Printer.h"
void
PlaceMarking::display (const class Printer& printer) const
{
  if (empty ())
    return;
  if (myPlace) {
    printer.print (myPlace->getName ());
    printer.delimiter (':')++;
  }
  bool comma = false;
  for (const_iterator i = myTokens.begin (); i != myTokens.end (); i++) {
    if (!i->second)
      continue;

    if (!comma) {
      if (myPlace)
	printer.linebreak ();
      comma = true;
    }
    else
      printer.delimiter (',');

    if (i->second > 1) {
      printer.print (i->second);
      printer.delimiter ('#');
    }
    i->first->display (printer);
  }
  if (myPlace)
    printer--;
}

card_t
PlaceMarking::sort (class PlaceMarking::card_value* histogram) const
{
  assert (histogram && !empty ());
  card_t total = 0;
  unsigned num = 0, pos = 0;
  for (const_iterator i = myTokens.begin ();
       i != myTokens.end ();
       i++, num++) {
    card_t card = i->second;
    if (!card) continue;
    if (total > CARD_T_MAX - card)
      return CARD_T_MAX;
    total += card;
    if (num) {
      for (unsigned low = 0, high = num; low != high; ) {
	pos = (low + high) >> 1;
	assert (pos < num && pos < high);
	if (histogram[pos].card > card)
	  high = pos;
	else if (++pos == high)
	  break;
	else {
	  assert (high > pos);
	  low = pos;
	}
      }
      memmove (histogram + pos + 1, histogram + pos,
	       (num - pos) * sizeof *histogram);
    }
    histogram[pos] = card_value (card, i->first);
  }
  assert (num == size ());
  return total;
}
