// Identifier type class -*- c++ -*-

#pragma implementation
#include "IdType.h"
#include "LeafValue.h"
#include "Constraint.h"
#include "Range.h"
#include "Printer.h"

/** @file IdType.C
 * Identifier data type
 */

class Value&
IdType::getFirstValue () const
{
  return *new class LeafValue (*this, 0);
}

class Value&
IdType::getLastValue () const
{
  return *new class LeafValue (*this, mySize - 1);
}

card_t
IdType::do_getNumValues () const
{
  assert (!myConstraint);
  return mySize;
}

card_t
IdType::convert (const class Value& value) const
{
  assert (&value.getType () == this);
  assert (!myConstraint);
  return card_t (static_cast<const class LeafValue&>(value));
}

class Value*
IdType::convert (card_t number) const
{
  assert (number < mySize);
  assert (!myConstraint);
  return new class LeafValue (*this, number);
}

#ifdef EXPR_COMPILE
# include "StringBuffer.h"

void
IdType::compileDefinition (class StringBuffer& out,
			   unsigned indent) const
{
  out.indent (indent);
  out.append ("unsigned");
}

void
IdType::do_compileConversion (class StringBuffer& out,
			      unsigned indent,
			      const char* value,
			      const char* number,
			      bool add) const
{
  out.indent (indent);
  out.append (number);
  out.append (add ? "+=" : "=");
  out.append (value);
  out.append (";\n");
}

#endif // EXPR_COMPILE

void
IdType::display (const class Printer& printer) const
{
  printer.printRaw ("id");
  printer.delimiter ('[');
  printer.print (mySize);
  printer.delimiter (']');
  assert (!myConstraint);
}
