// Variable stack map to be used with nested scoping -*- c++ -*-

#ifndef VARIABLESTACKMAP_H_
# define VARIABLESTACKMAP_H_
# pragma interface

# include <map>
# ifdef SLIST_LOC
#  include SLIST_LOC
# else
#  include <slist>
# endif
# include "util.h"

/** @file VariableStackMap.h
 * Nested symbol tables for variable names
 */

/** Variable stack map to be used with nested scoping */
class VariableStackMap
{
  /** Stack of variable definitions */
  typedef std::slist<class VariableDefinition*> Stack;
  /** Mapping from variable names to definition stacks */
  typedef std::map<const char*,Stack,struct ltstr> Map;

public:
  /** Constructor */
  VariableStackMap () : myMap (), myStack () {}
private:
  /** Copy constructor */
  VariableStackMap (const class VariableStackMap& old);
  /** Assignment operator */
  class VariableStackMap& operator= (const class VariableStackMap& old);
public:
  /** Destructor */
  ~VariableStackMap ();

  /** Push a variable definition on the stack
   * @param v		variable definition to be pushed on the stack
   */
  void push (class VariableDefinition* v);
  /** Get the topmost quantifier from the stack */
  class VariableDefinition* top () const { return *myStack.begin (); }
  /** Pop a quantifier from the stack */
  void pop ();
  /** Determine whether the container is empty */
  bool empty () const { return myStack.empty (); }
  /** Clear the container */
  void clear ();
  /** Find the topmost variable carrying the specified variable name
   * @param name	name being sought
   */
  class VariableDefinition* find (const char* name) const;

private:
  /** The mapping from names to variable definition stacks */
  Map myMap;
  /** Stack of all variable definitions */
  Stack myStack;
};

#endif // VARIABLESTACKMAP_H_
